/*
 * Copyright 2016 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.urmc.algo;

import at.jku.risc.stout.urmc.data.TermNode;
import at.jku.risc.stout.urmc.data.atom.Variable;

public class MatchProblem {
	private MatchEquation first, last;

	public boolean isEmpty() {
		return first == null;
	}

	public void add(MatchEquation eq) {
		eq.setNext(null);
		if (first == null) {
			first = last = eq;
		} else {
			last.setNext(eq);
			last = eq;
		}
	}

	public MatchEquation remove() {
		MatchEquation eq = first;
		first = eq.getNext();
		return eq;
	}

	public void apply(Variable fromVar, TermNode toTerm) {
		for (MatchEquation eq = first; eq != null; eq = eq.getNext())
			eq.apply(fromVar, toTerm);
	}

	public void apply(Substitution theta) {
		for (MatchEquation eq = first; eq != null; eq = eq.getNext())
			eq.apply(theta);
	}

	public MatchProblem copy() {
		MatchProblem cpy = new MatchProblem();
		if (first != null) {
			cpy.first = first.copy();
			cpy.last = cpy.first;
			for (; cpy.last.getNext() != null; cpy.last = cpy.last.getNext())
				cpy.last.setNext(cpy.last.getNext().copy());
		}
		return cpy;
	}

	public MatchEquation getFirst() {
		return first;
	}

	@Override
	public boolean equals(Object obj) {
		for (MatchEquation eq = first; eq != null; eq = eq.getNext())
			if (!eq.getLeft().equals(eq.getRight()))
				return false;
		return true;
	}

	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append('{');
		for (MatchEquation eq = first; eq != null; eq = eq.getNext())
			sb.append(eq).append(", ");
		if (!isEmpty())
			sb.setLength(sb.length() - 2);
		sb.append('}');
		return sb.toString();
	}
}
